@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c) 2024, Ryan Galloway (ryan@rsgalloway.com)
+# Copyright (c) 2024-2025, Ryan Galloway (ryan@rsgalloway.com)
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -33,24 +33,28 @@ __doc__ = """
 Contains common utility functions and classes.
 """
 
+import functools
 import glob
 import os
 import re
 import sys
+import time
 from ast import literal_eval
 from collections import OrderedDict
 
 import yaml
 
 from envstack import config
-from envstack.exceptions import CyclicalReference
 from envstack.node import AESGCMNode, Base64Node, EncryptedNode, FernetNode
 
+# default memoization cache timeout in seconds
+CACHE_TIMEOUT = 5
+
 # value for unresolvable variables
 null = ""
 
 # regular expression pattern for matching windows drive letters
-drive_letter_pattern = re.compile(r"(?P<sep>[:;])?(?P<drive>[A-Z]:[/\\])")
+drive_letter_pattern = re.compile(r"(?P<sep>[:;])?(?P<drive>[a-zA-Z]:[/\\])")
 
 # regular expression pattern for bash-like variable expansion
 variable_pattern = re.compile(
@@ -58,6 +62,25 @@ variable_pattern = re.compile(
 )
 
 
+def cache(func):
+    """Function decorator to memoize return data."""
+
+    cache_dict = {}
+
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        key = (args, tuple(kwargs.items()))
+        if key in cache_dict:
+            result, timestamp = cache_dict[key]
+            if time.time() - timestamp <= CACHE_TIMEOUT:
+                return result
+        result = func(*args, **kwargs)
+        cache_dict[key] = (result, time.time())
+        return result
+
+    return wrapper
+
+
 def clear_sys_path(var: str = "PYTHONPATH"):
     """
     Remove paths from sys.path that are in the given environment variable.
@@ -133,14 +156,16 @@ def split_windows_paths(path_str: str):
     tokens = [p.strip() for p in path_str.split(";") if p.strip()]
 
     for token in tokens:
-        # token is windows-style, insert a marker before drive letters
-        if re.match(r"^[A-Z]:[/\\]", token) or "\\" in token:
+        # windows-style path token; insert a marker before drive letters
+        if re.match(r"^[a-zA-Z]:[/\\]", token) or "\\" in token:
             modified = drive_letter_pattern.sub(lambda m: "|" + m.group("drive"), token)
             # split on the marker, then on colons that are not in drive-letters
             result += [
                 p
                 for part in modified.split("|")
-                for p in re.split(r"(?<![A-Z]):", part)
+                for p in re.split(
+                    r"(?<![a-zA-Z]):", part
+                )  # capture colons not in drive letters
                 if p
             ]
         else:
@@ -176,6 +201,57 @@ def dedupe_paths(path_str: str, platform: str = config.PLATFORM):
     return joiner.join(deduped)
 
 
+def detect_path(s: str):
+    """
+    Returns True if the given string looks like a filesystem path,
+    and False if it appears to be a URL or not a path.
+
+    This heuristic checks:
+      - That the string does not start with a URL scheme
+      - Windows drive letters or UNC paths
+      - POSIX absolute paths (starting with '/')
+      - Relative paths if they contain path separators or a file extension marker.
+
+    Note: Some valid file names (like "README") may be ambiguous.
+
+    :param s: The input string.
+    :returns: True if the string looks like a filesystem path.
+    """
+    s = s.strip()
+    if not s:
+        return False
+
+    # exclude variables
+    if s.startswith("$") or s.startswith("{") or s.endswith("}"):
+        return False
+
+    # exclude URL-like strings (scheme://...)
+    if re.match(r"^[a-zA-Z][a-zA-Z0-9+.-]*://", s):
+        return False
+
+    # windows: drive-letter (absolute or relative) or UNC path
+    if re.match(r"^[a-zA-Z]:", s) or s.startswith(r"\\\\"):
+        return True
+
+    # posix: absolute path starting with "/"
+    if s.startswith("/"):
+        return True
+
+    # check for delimiter-separated paths
+    if ";" in s or (":" in s and s.count(":") > 1):
+        return True
+
+    # check if the string contains a directory separator
+    if "/" in s or r"\\" in s:
+        return True
+
+    # check if the string has a dot (likely indicating a file extension)
+    # if re.search(r"(?<!^)\.", s):
+    #     return True
+
+    return False
+
+
 def dict_diff(dict1: dict, dict2: dict):
     """
     Compare two dictionaries and return their differences.
@@ -251,90 +327,155 @@ def get_stack_name(name: str = config.DEFAULT_NAMESPACE):
         raise ValueError("Invalid input type. Expected string, tuple, or list.")
 
 
-def evaluate_modifiers(expression: str, environ: dict = os.environ):
+def evaluate_modifiers(
+    expression: str,
+    environ: dict = os.environ,
+    parent: dict = {},
+    resolving: set = None,
+):
     """
-    Evaluates Bash-like variable expansion modifiers in a string, resolves
-    custom node types, and evaluates lists and dictionaries.
+    Evaluates Bash-like variable expansion modifiers.
 
-    Supported modifiers:
-    - values like "world" (no substitution)
-    - ${VAR} for direct substitution (empty string if unset)
-    - ${VAR:=default} to set and use a default value if unset
-    - ${VAR:?error message} to raise an error if the variable is unset or null
+    - ${VAR}              → empty string if unset
+    - ${VAR:=default}     → if unset/null, set to default and use it
+    - ${VAR:-default}     → if unset/null, use default (do not set)
+    - ${VAR:?message}     → error if unset/null
 
-    :param expression: The Bash-like string, e.g.,
-        "${VAR:=default}/path", "${VAR}/path", or "${VAR:?error message}"
-    :return: The resulting evaluated string with all substitutions applied.
-    :raises CyclicalReference: If a cyclical reference is detected.
-    :raises ValueError: If a variable is undefined and has the :? syntax with an
-        error message.
+    :param expression: The string to evaluate.
+    :param environ: The environment variables to use for substitution.
+    :param parent: Parent environment variables to use for substitution.
+    :param resolving: A set of currently resolving variables to prevent cycles.
+    :returns: The evaluated string with variables substituted.
+    :raises ValueError: If a variable is not set and a message is provided.
     """
+    if resolving is None:
+        resolving = set()
 
     def sanitize_value(value):
-        """Sanitize the value before returning it."""
-        # HACK: remove trailing curly braces if they exist
-        if type(value) is str and value.endswith("}") and not value.startswith("${"):
+        # sanitize the value to ensure it is a string or a path
+        if (
+            isinstance(value, str)
+            and value.endswith("}")
+            and not value.startswith("${")
+        ):
             return value.rstrip("}")
-        # sanitize path-like values
-        elif type(value) is str and ":" in value and ("/" in value or "\\" in value):
+        elif isinstance(value, str) and detect_path(value):
             return dedupe_paths(value)
         return value
 
+    def is_template(s: str) -> bool:
+        # Check if the string is a template variable like ${VAR}
+        return isinstance(s, str) and bool(variable_pattern.search(s))
+
+    def non_empty(s: str) -> bool:
+        # non-empty string, not just whitespace
+        return isinstance(s, str) and s != ""
+
+    def is_literal(s: str) -> bool:
+        # non-empty, does not contain ${...}
+        return isinstance(s, str) and s != "" and not is_template(s)
+
     def substitute_variable(match):
-        """Substitute a variable match with its value."""
+        # extract variable name, operator, and argument from the match
         var_name = match.group(1)
-        operator = match.group(2)
-        argument = match.group(3)
-        override = os.getenv(var_name, null)
-        value = str(environ.get(var_name, override))
+        operator = match.group(2)  # '=', '-', '?', or None
+        argument = match.group(3)  # may be None
+        parent_value = parent.get(var_name, "")
+        override = os.getenv(var_name, parent_value)
+        current = str(
+            environ.get(var_name, parent_value or "")
+        )  # current value we have
         varstr = "${%s}" % var_name
 
-        # check for self-referential values
-        is_recursive = value and varstr in value
+        # cycle / self-reference guard
+        if var_name in resolving:
+            if operator in ("=", "-"):
+                # if os/parent provides a value, that wins over the default
+                if override:
+                    return str(override)
+                default = evaluate_modifiers(argument or "", environ, parent, resolving)
+                if operator == "=":
+                    environ[var_name] = default
+                return str(default)
+            elif operator == "?":
+                msg = argument or f"{var_name} is not set"
+                raise ValueError(msg)
+            else:  # plain ${VAR}
+                return str(override or "")
 
-        # handle recursive references
-        if is_recursive and override:
-            value = value.replace(varstr, override)
-        else:
-            value = value.replace(varstr, "")
+        resolving.add(var_name)
+        try:
+            # replace literal self-references inside current (best-effort)
+            if current and varstr in current:
+                current = current.replace(varstr, override or "")
 
-        # ${VAR:=default} or ${VAR:-default}
-        if operator in ("=", "-"):
-            if override:
-                value = override
-            elif variable_pattern.search(value) or value is None:
-                value = evaluate_modifiers(argument, environ)
+            # defaults branch (:=, :-)
+            if operator in ("=", "-"):
+                # 1) explicit env/parent wins if non-empty (bash: considered "set")
+                if non_empty(override):
+                    return str(override)
+
+                # 2) compute the effective value of the current entry
+                if is_template(current):
+                    eff = evaluate_modifiers(current, environ, parent, resolving)
+                else:
+                    eff = current or ""
+
+                # 3) if effective value is non-empty, do NOT take default
+                if non_empty(eff):
+                    return str(eff)
+
+                # 4) unset or null,  use default (and assign for :=)
+                default = evaluate_modifiers(argument or "", environ, parent, resolving)
+                if operator == "=":
+                    environ[var_name] = default
+                return str(default)
+
+            # error branch (:?)
+            elif operator == "?":
+                if not current:
+                    msg = argument or f"{var_name} is not set"
+                    raise ValueError(msg)
+                # resolve if it contains vars
+                return (
+                    evaluate_modifiers(current, environ, parent, resolving)
+                    if variable_pattern.search(current)
+                    else current
+                )
+
+            elif operator is None:
+                if is_literal(current):
+                    return current  # file literal wins
+                if is_template(current):
+                    return str(evaluate_modifiers(current, environ, parent, resolving))
+                # current is unset/empty, use parent/os if present, else empty
+                return str(override or "")
+
+            # simple ${VAR}
             else:
-                value = value or argument
-        # ${VAR:?error message}
-        elif operator == "?":
-            if not value:
-                error_message = argument if argument else f"{var_name} is not set"
-                raise ValueError(error_message)
-        # handle recursive references
-        elif variable_pattern.search(value):
-            value = evaluate_modifiers(value, environ)
-        # handle simple ${VAR} substitution
-        elif operator is None:
-            value = value or ""
+                value = current or override or ""
+                if variable_pattern.search(value):
+                    value = evaluate_modifiers(value, environ, parent, resolving)
+                return str(value)
 
-        return value
+        finally:
+            resolving.discard(var_name)
 
     try:
-        # substitute all matches in the expression
+        if isinstance(expression, (int, float, bool)):
+            expression = str(expression)
+
         result = variable_pattern.sub(substitute_variable, expression)
 
-        # evaluate any remaining modifiers, eg. ${VAR:=${FOO:=bar}}
         if variable_pattern.search(result):
-            result = evaluate_modifiers(result, environ)
+            result = evaluate_modifiers(result, environ, parent, resolving)
 
-    # detect recursion errors
     except RecursionError:
-        raise CyclicalReference(f"Cyclical reference detected in {expression}")
+        # treat cycles as empty string instead of crashing (like bash)
+        result = ""
 
-    # evaluate other data types
-    # TODO: find a better way to evaluate other data types
     except TypeError:
+        # node/list/dict handlers
         if isinstance(expression, AESGCMNode):
             result = expression.resolve(env=environ)
         elif isinstance(expression, Base64Node):
@@ -344,10 +485,10 @@ def evaluate_modifiers(expression: str, environ: dict = os.environ):
         elif isinstance(expression, FernetNode):
             result = expression.resolve(env=environ)
         elif isinstance(expression, list):
-            result = [(evaluate_modifiers(v, environ)) for v in expression]
+            result = [evaluate_modifiers(v, environ, parent) for v in expression]
         elif isinstance(expression, dict):
             result = {
-                k: (evaluate_modifiers(v, environ)) for k, v in expression.items()
+                k: evaluate_modifiers(v, environ, parent) for k, v in expression.items()
             }
         else:
             result = expression
@@ -523,8 +664,7 @@ def dump_yaml(file_path: str, data: dict, unquote: bool = True):
 
     partitioned_data = partition_platform_data(data)
 
-    # write the platform partidioned data to the env file,
-    # add shebang to make it executable
+    # write the platform partidioned data add shebang
     with open(file_path, "w") as file:
         file.write("#!/usr/bin/env envstack\n")
         if data.get("include"):
@@ -586,7 +726,10 @@ def partition_platform_data(data: dict):
                 common_keys.append(key)
 
     # build a new all dict for common items
-    new_all = {"<<": "*all"}  # avoids syntax errors when no vars are present
+    if common_keys or all_platform_keys or data.get("all"):
+        new_all = {}
+    else:
+        new_all = {"<<": "*all"}  # avoids syntax errors when no vars are present
     for k in common_keys:
         if k in data["all"]:
             new_all[k] = data["all"][k]
